@@ -326,7 +326,9 @@ my @builtin_chords =
  "Amaj7"       => [  N, 0, 2, 1, 2, 0,	 1 ],
  "Am"	       => [  N, 0, 2, 2, 1, 0,	 1 ],
  "Amin"	       => [  N, 0, 2, 2, 1, 0,	 1 ],
+ "A/C#"	       => [  N, 4, 2, 2, 2, 0,	 1 ],
  "A/D"	       => [  N, N, 0, 0, 2, 2,	 1 ],
+ "A/E"	       => [  0, 0, 2, 2, 2, 0,	 1 ],
  "A/F#"	       => [  2, 0, 2, 2, 2, 0,	 1 ],
  "A/G#"	       => [  4, 0, 2, 2, 2, 0,	 1 ],
 
@@ -979,6 +981,11 @@ sub transpose {
     $mod++ if $r =~ s/(is|\#)$//;
     warn("WRONG NOTE: '$c' '$r' '$rest'") unless defined $notes2canon{$r};
     $r = ($notes2canon{$r} + $mod + $xpose) % 12;
+
+    if ( $rest =~ m;^(.*)/(.*); ) {
+	$rest = $1 . "/" . transpose( $2, $xpose );
+    }
+
     return ( $xpose > 0 ? \@notes_sharp : \@notes_flat )->[$r] . $rest;
 }
 
