@@ -24,6 +24,12 @@ This is the current built-in configuration file, showing all settings.
   // This is a relaxed JSON document, so comments are possible.
   
   {
+      // Includes. These are processed first, before the rest of
+      // the config file.
+      //
+      // "include" takes a list of either filenames or preset names.
+      // "include" : [ "modern1", "lib/mycfg.json" ],
+  
       // General settings, to be changed by legacy configs and
       // command line.
       "settings" : {
@@ -164,7 +170,8 @@ This is the current built-in configuration file, showing all settings.
 	// This opens a margin for margin labels.
 	"labels" : {
 	    // Margin width. Default is 0 (no margin labels).
-	    "width" : 0,
+            // "auto" will automatically reserve a margin if labels are used.
+	    "width" : "auto",
 	    // Alignment for the labels. Default is left.
 	    "align" : "left",
 	},
@@ -173,6 +180,7 @@ This is the current built-in configuration file, showing all settings.
   	// Value is the column position.
   	// "chordscolumn" : 400,
   	"chordscolumn" :  0,
+	"capoheading" : "%{capo|Capo: %{}}",
   
 	// A {titles: left} may conflict with customized formats.
 	// Set to non-zero to ignore the directive.
@@ -299,6 +307,7 @@ This is the current built-in configuration file, showing all settings.
 
 =cut
 
+use File::Spec;
 use JSON::PP ();
 
 sub hmerge($$$);
@@ -533,6 +542,24 @@ sub add_config {
 	    warn("$file: pdf.formats.default.toc-title is obsolete, use toc.title instead\n");
 	}
 
+	# If there are includes, process these first.
+	if ( exists $new->{include} ) {
+	    my ( $vol, $dir, undef ) = File::Spec->splitpath($file);
+	    foreach my $c ( @{ delete $new->{include} } ) {
+		# Check for resource names.
+		if ( $c !~ m;[/.]; ) {
+		    my $t = ::findlib( "config/".lc($c).".json" );
+		    $c = $t if $t;
+		}
+		elsif ( $dir ne ""
+			&& !File::Spec->file_name_is_absolute($c) ) {
+		    # Prepend dir of the caller, if needed.
+		    $c = File::Spec->catpath( $vol, $dir, $c );
+		}
+		$cfg = add_config( $cfg, $options, $c, $pp );
+	    }
+	}
+
 	# Merge final.
 	$cfg = hmerge( $cfg, $new, "" );
 	close($fd);
