@@ -153,6 +153,15 @@ sub generate_song {
 
     my $sb = $s->{body};
 
+    # Load song chords, if any.
+    App::Music::ChordPro::Chords::reset_song_chords();
+    if ( $s->{define} ) {
+	foreach ( @{ $s->{define} } ) {
+	    App::Music::ChordPro::Chords::add_song_chord
+		( $_->{name}, $_->{base}, $_->{frets}, $_->{fingers} );
+	}
+    }
+
     # set_columns needs these, set provisional values.
     $ps->{_leftmargin}  = $ps->{marginleft};
     $ps->{_rightmargin} = $ps->{marginright};
@@ -162,7 +171,24 @@ sub generate_song {
     $chordscol    = $ps->{chordscolumn};
     $lyrics_only  = $::config->{settings}->{'lyrics-only'};
     $chordscapo   = $s->{meta}->{capo};
-    $ps->{_indent} = $ps->{labels}->{width};
+
+    if ( $ps->{labels}->{width} eq "auto" ) {
+	if ( $s->{labels} && @{ $s->{labels} } ) {
+	    my $longest = 0;
+	    my $ftext = $fonts->{text};
+	    for ( @{ $s->{labels} } ) {
+		my $t = $ftext->{font}->width("$_    ") * $ftext->{size};
+		$longest = $t if $t > $longest;
+	    }
+	    $ps->{_indent} = $longest;
+	}
+	else {
+	    $ps->{_indent} = 0;
+	}
+    }
+    else {
+	$ps->{_indent} = $ps->{labels}->{width};
+    }
 
     my $fail;
     for my $item ( @{ SIZE_ITEMS() } ) {
@@ -198,7 +224,7 @@ sub generate_song {
 
     my $st = $s->{settings}->{titles} || $::config->{settings}->{titles};
     if ( defined($st)
-	 && ! $pr->{'titles-directive-ignore'} ) {
+	 && ! $ps->{'titles-directive-ignore'} ) {
 	my $swap = sub {
 	    my ( $from, $to ) = @_;
 	    for my $class ( qw( default title first ) ) {
@@ -615,7 +641,12 @@ sub generate_song {
 
 	    # Draw background.
 	    my $bgcol = $ftext->{background};
-	    $bgcol ||= "#E5E5E5" if $elt->{type} eq "comment";
+	    if ( $elt->{type} eq "comment" ) {
+		# Default to grey.
+		$bgcol ||= "#E5E5E5";
+		# Since we default to grey, we need a way to cancel it.
+		undef $bgcol if $bgcol eq "none";
+	    }
 	    if ( $bgcol ) {
 		$pr->rectxy( $x + $indent - 2, $y + 2,
 			     $x + $indent + $w + 2, $y - $vsp, 3, $bgcol );
@@ -627,7 +658,8 @@ sub generate_song {
 		$x0 += 0.25;	# add some offset for the box
 		$pr->rectxy( $x0 + $indent, $y + 1,
 			     $x0 + $indent + $w + 1, $y - $vsp + 1,
-			     0.5, undef, "black" );
+			     0.5, undef,
+			     $ftext->{color} || "black" );
 	    }
 
 	    songline( $elt, $x0, $y, $ps, song => $s, indent => $indent );
@@ -855,7 +887,7 @@ sub generate_song {
 		$cells += $grid_margin->[1] = $v[3] if $v[3];
 		$grid_margin->[2] = $cells;
 	    }
-	    elsif ( $elt->{name} eq "tag" ) {
+	    elsif ( $elt->{name} eq "label" ) {
 		$i_tag = $elt->{value};
 	    }
 	    # Arbitrary config values.
@@ -1050,7 +1082,7 @@ sub songline {
 	if ( $chordscol && $chord ne "" ) {
 
 	    if ( $chordscapo ) {
-		$pr->text("Capo: " . $chordscapo,
+		$pr->text(fmt_subst( $opts{song}, $ps->{capoheading} ),
 			  $chordsx,
 			  $ytext + $ftext->{size} *
 			      $ps->{spacing}->{chords},
@@ -1107,7 +1139,7 @@ sub songline {
 				 );
 		$xt0 = $pr->text( $post, $xt0, $ychord, $fchord );
 	    }
-	    elsif ( $info->{dproot} || $_ > 0 ) {
+	    elsif ( $chord) {
 		$xt0 = $pr->text( $pre.$chord.$post, $x, $ychord, $fchord );
 	    }
 	    else {
@@ -1422,8 +1454,8 @@ sub tocline {
 
     my $ann = $pr->{pdfpage}->annotation;
     $ann->link($elt->{page});
-    $ann->rect( $ps->{_leftmargin}, $y0 - $ftoc->{size},
-		$ps->{__marginright}, $y0 );
+    $ann->rect( $ps->{_leftmargin}, $y0 - $ftoc->{size} * $ps->{spacing}->{toc},
+		$ps->{__rightmargin}, $y0 );
     ####CHECK MARGIN RIGHT
 }
 
@@ -2112,17 +2144,23 @@ sub init_font {
     my $font = $ps->{fonts}->{$ff};
     if ( $font->{file} ) {
 	if ( $font->{file} =~ /\.[ot]tf$/ ) {
-	    $font->{font} =
-	      $fontcache{$font->{file}} ||=
-	      $self->{pdf}->ttfont( $font->{file},
-				    -dokern => 1 );
+	    eval {
+		$font->{font} =
+		  $fontcache{$font->{file}} ||=
+		    $self->{pdf}->ttfont( $font->{file},
+					  -dokern => 1 );
+	    }
+	    or warn("Cannot load font: ", $font->{file}, "\n");
 	}
 	elsif ( $font->{file} =~ /\.pf[ab]$/ ) {
-	    $font->{font} =
-	      $fontcache{$font->{file}} ||=
-	      $self->{pdf}->psfont( $font->{file},
-				    -afmfile => $font->{metrics},
-				    -dokern  => 1 );
+	    eval {
+		$font->{font} =
+		  $fontcache{$font->{file}} ||=
+		    $self->{pdf}->psfont( $font->{file},
+					  -afmfile => $font->{metrics},
+					  -dokern  => 1 );
+	    }
+	    or warn("Cannot load font: ", $font->{file}, "\n");
 	}
 	else {
 	    $font->{font} =
@@ -2131,9 +2169,12 @@ sub init_font {
 	}
     }
     else {
-	$font->{font} =
-	  $fontcache{"__core__".$font->{name}} ||=
-	    $self->{pdf}->corefont( $font->{name}, -dokern => 1 );
+	eval {
+	    $font->{font} =
+	      $fontcache{"__core__".$font->{name}} ||=
+		$self->{pdf}->corefont( $font->{name}, -dokern => 1 );
+	}
+	or warn("Cannot load font: ", $font->{file}, "\n");
     }
 
     unless ( $font->{font} ) {
