@@ -33,12 +33,15 @@ my @used_chords;
 
 # Chorus lines, if any.
 my @chorus;
+my $chorus_xpose = 0;
 
 # Keep track of unknown chords, to avoid dup warnings.
 my %warned_chords;
 
 my $re_meta;			# for metadata
 
+my @labels;			# labels used
+
 # Normally, transposition and subtitutions are handled by the parser.
 my $no_transpose;		# NYI
 my $no_substitute;
@@ -87,6 +90,7 @@ sub parse_song {
     %warned_chords = ();
     undef $chordtype;
     App::Music::ChordPro::Chords::reset_song_chords();
+    @labels = ();
 
     # Build regex for the known metadata items.
     if ( $::config->{metadata}->{keys} ) {
@@ -157,6 +161,10 @@ sub parse_song {
     do_warn("Unterminated context in song: $in_context")
       if $in_context;
 
+    if ( @labels ) {
+	$song->{labels} = \@labels;
+    }
+
     my $diagrams;
     if ( exists($song->{settings}->{diagrams} ) ) {
 	$diagrams = $song->{settings}->{diagrams};
@@ -170,7 +178,7 @@ sub parse_song {
 	 && defined($chordtype)
 	 && $chordtype =~ /^[RN]$/ ) {
 	$diag->{orig} = "(End of Song)";
-	do_warn("Chord diagrams suppressed for Nasville/Roman chords");
+	do_warn("Chord diagrams suppressed for Nashville/Roman chords");
 	$diagrams = "none";
     }
 
@@ -208,8 +216,10 @@ sub add {
     push( @{$song->{body}},
 	  { context => $in_context,
 	    @_ } );
-    push( @chorus, { context => $in_context, @_ } )
-      if $in_context eq "chorus";
+    if ( $in_context eq "chorus" ) {
+	push( @chorus, { context => $in_context, @_ } );
+	$chorus_xpose = $xpose;
+    }
 }
 
 sub chord {
@@ -414,14 +424,15 @@ sub directive {
 	}
 	elsif ( $arg && $arg ne "" ) {
 	    $self->add( type  => "set",
-			name  => "tag",
+			name  => "label",
 			value => $arg );
+	    push( @labels, $arg );
 	}
 	else {
 	    do_warn("Garbage in start_of_$1: $arg (ignored)\n")
 	      if $arg;
 	}
-	@chorus = () if $in_context eq "chorus";
+	@chorus = (), $chorus_xpose = 0 if $in_context eq "chorus";
 	return 1;
     }
     if ( $dir =~ /^end_of_(\w+)$/ ) {
@@ -438,7 +449,7 @@ sub directive {
 	$self->add( type => "rechorus",
 		    @chorus
 		    ? ( "chorus" => App::Music::ChordPro::Config::clone(\@chorus),
-			"transpose" => $xpose )
+			"transpose" => $xpose - $chorus_xpose )
 		    : (),
 		  );
 	return 1;
@@ -540,6 +551,9 @@ sub directive {
 	if ( $xpose && $1 eq "key" ) {
 	    $arg = App::Music::ChordPro::Chords::transpose( $arg, $xpose );
 	}
+	if ( $1 eq "capo" && $song->{meta}->{capo} ) {
+	    do_warn("Multiple capo settings may yield surprising results.");
+	}
 	push( @{ $song->{meta}->{$1} }, $arg );
 	return 1;
     }
@@ -552,6 +566,9 @@ sub directive {
 	    if ( $xpose && $key eq "key" ) {
 		$val = App::Music::ChordPro::Chords::transpose( $val, $xpose );
 	    }
+	    if ( $key eq "capo" && $song->{meta}->{capo} ) {
+		do_warn("Multiple capo settings may yield surprising results.");
+	    }
 	    if ( $re_meta && $key =~ $re_meta ) {
 		# Known.
 		push( @{ $song->{meta}->{$key} }, $val );
@@ -982,7 +999,7 @@ sub _transpose {
 
     if ( $item->{type} eq "gridline" ) {
 	foreach ( @{ $item->{tokens} } ) {
-	    return unless $_->{class} eq "chord";
+	    next unless $_->{class} eq "chord";
 	    $_->{chord} = $self->xpchord( $_->{chord}, $xpose );
 	}
 	if ( $item->{margin} && exists $item->{margin}->{chords} ) {
